/**********************************************/
/* mqprcobj.hpp ... Process obj declarations. */
/* Part of SoftWoehr Library for IBM MQSeries */
/* Copyright *C* 1999 Jack J. Woehr           */
/* PO Box 51, Golden, Colorado 80402-0051     */
/* http://www.well.com/user/jax/SoftWoehr     */
/* jwoehr@ibm.net jax@well.com                */
/* ********************************************/
/* This is open source software. Please see   */
/* file license.txt. THERE IS NO WARRANTY.    */
/**********************************************/
#ifndef SW_MQPRCOBJ_HPP
#define SW_MQPRCOBJ_HPP

#ifndef SW_MQOBJ_HPP
   #include "mqobj.hpp"
#endif // SW_MQOBJ_HPP

#ifndef SW_MQINQ_HPP
   #include "mqinq.hpp"
#endif // SW_MQINQ_HPP

namespace SoftWoehr {

/** Represents a queue manager as an object */
class MQProcessDefinition : public MQObject
   {
   public   :

      /** Create an MQProcessDefinition with the descriptor already set to Queue */
      MQProcessDefinition();

      /** Dtor */
      virtual ~MQProcessDefinition();

      /** Open this object in style of superclass.
        * This overload serves merely to keep the very picky Sun
        * Sun CC 5.0 * compiler from complaining about name hiding.
        */
      virtual MQObject & open ( MQQueueManager & queue_manager
                              , const Descriptor& descriptor
                              , const Options   & options
                              )
        {
        return MQObject::open( queue_manager
                             , descriptor
                             , options
                             )
                             ;
        }

      /** Open a given queue manager as an object. */
      virtual MQProcessDefinition &
         open( MQQueueManager & queue_manager
             , const std::string & process_name
             , const MQObject::Options & options
             )
             ;

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* The code within these blocks was autogenerated.   */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/** Character attribute factory for MQCA_APPL_ID */
static MQInquiry::character_attribute *
   mqca_appl_id(void);

/** Character attribute factory for MQCA_ENV_DATA */
static MQInquiry::character_attribute *
   mqca_env_data(void);

/** Character attribute factory for MQCA_PROCESS_DESC */
static MQInquiry::character_attribute *
   mqca_process_desc(void);

/** Character attribute factory for MQCA_PROCESS_NAME */
static MQInquiry::character_attribute *
   mqca_process_name(void);

/** Character attribute factory for MQCA_USER_DATA */
static MQInquiry::character_attribute *
   mqca_user_data(void);

/** Integer attribute factory for MQIA_APPL_TYPE */
static MQInquiry::integer_attribute *
   mqia_appl_type(void);

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* End of autogenerated code.                        */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/** Compose an inquiry which applies to any proc def.
  * Ask everything you can MQINQ about a proc def
  */
static MQInquiry * inquiry_any_process_definition(void);

};                          /* class MQProcessDefinition : public MQObject*/

} // namespace SoftWoehr

#endif                                 /* SW_MQPRCOBJ_HPP           */
